--
-- Create Date:    16:20:03 08/24/2010 
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity compact_ad is
    Port ( clk_50 : in  STD_LOGIC;
           spi_miso : in  STD_LOGIC;
           ad_a : out  STD_LOGIC_VECTOR (13 downto 0);
           ad_b : out  STD_LOGIC_VECTOR (13 downto 0);
           new_values : out  STD_LOGIC;
           amp_cs : out  STD_LOGIC;
			  ad_buzy : out  STD_LOGIC;
           ad_conv : out  STD_LOGIC;
           spi_sck : out  STD_LOGIC;
           spi_mosi : out  STD_LOGIC;
           spi_ss_b : out  STD_LOGIC;
           sf_ce0 : out  STD_LOGIC;
           fpga_init_b : out  STD_LOGIC);
end compact_ad;

architecture Behavioral of compact_ad is
	signal s: integer range 0 to 4095:= 0; -- sequence and sampling rate counter
	signal init_counter: integer range 0 to 7:= 0; 
	signal amp_clk: STD_LOGIC := '0';	--amp clock generation
	signal ad_clk: STD_LOGIC := '0';	--ad clock generation
	signal tmp: STD_LOGIC_VECTOR (13 downto 0):="00000000000000";
--*******************************************************************
	constant clk_periods_per_samp 	: integer range 0 to 4095:= 250;
	constant nbr_of_inits 	: integer range 0 to 7:= 3;	
	constant gain : STD_LOGIC_VECTOR (7 downto 0):="00010001"; 
--********************************************************************
-- main program
begin
	process (clk_50) 
	begin
		if clk_50'event and clk_50='1' then
			if s < clk_periods_per_samp -1 then
				s <= s+1;
			else
				s <= 0;		
			end if;
			if (init_counter < nbr_of_inits)and(s=clk_periods_per_samp -1) then
				init_counter <= init_counter+1;
			else
				init_counter <= init_counter;
			end if;
		end if;
	end process;
--end main
--**************************************************
--***********************************
--internal clk for init phase	
	process (clk_50,s)
	begin
		if clk_50'event and clk_50='1' then
			if (s=0)or(s>47) then
				amp_clk <= '0';
			elsif 	((s>1)and(s<6))or((s>7)and(s<12))or((s>13)and(s<18))or((s>19)and(s<24))or
						((s>25)and(s<30))or((s>31)and(s<36))or((s>37)and(s<42))or((s>43)and(s<48))then
				amp_clk <= '1';
			else
				amp_clk <= '0';
			end if; 
		end if;
	end process;
--end of	internal clk for init phase	
--**************************************************				

--normal phase

--***************************************************
--clk for normal phase
	process (clk_50,s)
	begin
		if clk_50'event and clk_50='1' then
			if (s=2)or(s=7)or(s=12)or(s=17)or(s=22)or(s=27)or(s=32)or(s=37)or(s=42)or(s=47)or 
				(s=52)or(s=57)or(s=62)or(s=67)or(s=72)or(s=77)or(s=82)or(s=87)or(s=92)or(s=97)or 
				(s=102)or(s=107)or(s=112)or(s=117)or(s=122)or(s=127)or(s=132)or(s=137)or(s=142)or(s=147)or
				(s=152)or(s=157)or(s=162)or(s=167) then
				ad_clk <= '1';
			else				
				ad_clk <= '0';
			end if;
		end if;
	end process;
--end of clk for normal phase
--***************************************************
--start of serial (from ad) to parallel temporary register	
	process (clk_50,s,init_counter)
	begin
		if (clk_50='1' and clk_50'event)and(init_counter = nbr_of_inits) then
			if (s=0)or(s=84) then tmp <= "00000000000000"; end if;
			if (spi_miso='1')and(s=15)then 	tmp(13)<= '1'; end if;
			if (spi_miso='1')and(s=20)then 	tmp(12)<= '1'; end if;
			if (spi_miso='1')and(s=25)then 	tmp(11)<= '1'; end if;
			if (spi_miso='1')and(s=30)then 	tmp(10)<= '1'; end if;
			if (spi_miso='1')and(s=35)then 	tmp(9)<= '1'; end if;
			if (spi_miso='1')and(s=40)then 	tmp(8)<= '1'; end if;
			if (spi_miso='1')and(s=45)then 	tmp(7)<= '1'; end if;
			if (spi_miso='1')and(s=50)then 	tmp(6)<= '1'; end if;
			if (spi_miso='1')and(s=55)then 	tmp(5)<= '1'; end if;
			if (spi_miso='1')and(s=60)then 	tmp(4)<= '1'; end if;
			if (spi_miso='1')and(s=65)then  tmp(3)<= '1'; end if;
			if (spi_miso='1')and(s=70)then  tmp(2)<= '1'; end if;
			if (spi_miso='1')and(s=75)then  tmp(1)<= '1'; end if; 
			if (spi_miso='1')and(s=80)then  tmp(0)<= '1'; end if; 
			
			if (spi_miso='1')and(s=95)then 	tmp(13)<= '1'; end if;
			if (spi_miso='1')and(s=100)then 	tmp(12)<= '1'; end if;
			if (spi_miso='1')and(s=105)then 	tmp(11)<= '1'; end if;
			if (spi_miso='1')and(s=110)then 	tmp(10)<= '1'; end if;
			if (spi_miso='1')and(s=115)then 	tmp(9)<= '1'; end if;
			if (spi_miso='1')and(s=120)then 	tmp(8)<= '1'; end if;
			if (spi_miso='1')and(s=125)then 	tmp(7)<= '1'; end if;
			if (spi_miso='1')and(s=130)then 	tmp(6)<= '1'; end if;
			if (spi_miso='1')and(s=135)then 	tmp(5)<= '1'; end if;
			if (spi_miso='1')and(s=140)then 	tmp(4)<= '1'; end if;
			if (spi_miso='1')and(s=145)then 	tmp(3)<= '1'; end if;
			if (spi_miso='1')and(s=150)then 	tmp(2)<= '1'; end if;
			if (spi_miso='1')and(s=155)then 	tmp(1)<= '1'; end if;
			if (spi_miso='1')and(s=160)then 	tmp(0)<= '1'; end if;	
		end if;
	end process;		
--end of serial (from ad) to parallel temporary register	
--****************************************************
--***********************************************************
--generat outputs		
--****************************************************
--start of transfer from temporary to output registers for both channels
	process (clk_50) 
	begin
		if clk_50='1' and clk_50'event then
			if s=82 then
				ad_a <= tmp;
			end if;
			if s=162 then
				ad_b <= tmp;
			end if;
		end if;
	end process;
--end of transfer from temporary to output registers for both channels
--****************************************************	
--Set gain of preamplifier (set to -1)
--Spartan-3E Starter Kit User Guide table 10.2 (B3,B2,B1,B0,A3,A2,A1,A0)	
	amp_cs <= '0' when (s>0)and(s<50)and(init_counter<nbr_of_inits)and(init_counter>0) else '1';
	spi_mosi <= '1' when  (((gain(7)='1')and(s>0)and(s<7))or
								((gain(6)='1')and(s>6)and(s<13))or
								((gain(5)='1')and(s>12)and(s<19))or
								((gain(4)='1')and(s>18)and(s<25))or
								((gain(3)='1')and(s>24)and(s<31))or
								((gain(2)='1')and(s>30)and(s<37))or
								((gain(1)='1')and(s>36)and(s<43))or
								((gain(0)='1')and(s>42)and(s<49)))and(init_counter<nbr_of_inits)and(init_counter>0)
						else '0';
--end of init specific signal generation
--*********************************************************************
	ad_buzy <= '1' when s<180 else '0';
	
	process (init_counter,amp_clk,ad_clk) 
	begin
		if (init_counter = 0)then
			spi_sck <= '0';
		elsif (init_counter < nbr_of_inits)then
			spi_sck <= amp_clk;	
		else
			spi_sck <= ad_clk;
		end if;
	end process;

   new_values <= '1' when (s=180)and(init_counter = nbr_of_inits) else '0';
   ad_conv <= '1' when (s=1)and(init_counter = nbr_of_inits) else '0';

--***************************************************--*************************
-- disable other components
	spi_ss_b <= '1';
	sf_ce0 <= '1';
	fpga_init_b <= '1';
--*************************
end Behavioral;



























