--
-- Create Date:    13:33:05 09/01/2010 
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity compact_lcd_driver is
    Port ( clk_50 : in  STD_LOGIC;
           update_lcd : in  STD_LOGIC;
           ascii : in  STD_LOGIC_VECTOR (7 downto 0);
           next_char : out  STD_LOGIC;
           lcd_rw : out  STD_LOGIC;
           lcd_e : out  STD_LOGIC;
           lcd_rs : out  STD_LOGIC;
           nibble_to_lcd : out  STD_LOGIC_VECTOR (3 downto 0));
end compact_lcd_driver;

architecture Behavioral of compact_lcd_driver is

--**************************************************************************************
signal set_timer_t1 		: STD_LOGIC;
signal set_timer_t2 		: STD_LOGIC;
signal set_timer_t3 		: STD_LOGIC;
signal tim_done 			: STD_LOGIC;
signal s						: integer range 0 to 15;
signal byte_done 			: STD_LOGIC;
signal byte 				: STD_LOGIC_VECTOR (7 downto 0):="00000000";
signal enable_byte_write: STD_LOGIC;
signal timer_value_t1 	: STD_LOGIC_VECTOR (19 downto 0):="00000000000000000000";
signal timer_value_t2 	: STD_LOGIC_VECTOR (19 downto 0):="00000000000000000000";
signal timer_value_t3 	: STD_LOGIC_VECTOR (19 downto 0):="00000000000000000000";
signal q						: STD_LOGIC_VECTOR (5 downto 0):="000000";
signal t						: STD_LOGIC_VECTOR (19 downto 0):="00000000000000000000";

constant T_15ms   : STD_LOGIC_VECTOR (19 downto 0):="10110111000110110001";
constant T_4_1ms  : STD_LOGIC_VECTOR (19 downto 0):="00110010000011001001";
constant T_1_64ms : STD_LOGIC_VECTOR (19 downto 0):="00010100000001010001";
constant T_100us  : STD_LOGIC_VECTOR (19 downto 0):="00000001001110001001";
constant T_40us   : STD_LOGIC_VECTOR (19 downto 0):="00000000011111010001";
constant T_1us    : STD_LOGIC_VECTOR (19 downto 0):="00000000000000110011";
--*******************************************************
begin
	process(clk_50)
	begin
		if ((clk_50'event) and (clk_50='1')) then
--*****************************************************
--lcd_state_machine is
			if q="000000" then
				q <= "000001";
			elsif (q="000110")and(update_lcd='1') then
				q <= "000111";
			elsif (q="101000")and(byte_done='1') then
				q <= "000110";
			elsif (q /= "000110")and(byte_done='1') then
				q <= q+1;
			else
				q <=q;
			end if;
--*****************************************************
--write_byte_control
			if s=0 then								-- wait for write byte
				if enable_byte_write='1' then
					s <= 1;
				else
					s <= 0;
				end if;
			elsif	s=1 then							-- set pre write timer
					s <= 2;			
			elsif s=2 then								-- wait for pre write timer
				if tim_done='1' then
					s <= 3;
				else
					s <= 2;
				end if;
			elsif s=3 then								-- put first nibble out
					s <= 4;
			elsif s=4 then								-- set first nibble write timer
					s <= 5;
			elsif s=5 then								-- enable first nibble write
				if tim_done='1' then
					s <= 6;
				else
					s <= 5;
				end if;
			elsif s=6 then								-- set inter nibble timer
					s <= 7;
			elsif s=7 then								-- wait for inter nibble timer
				if tim_done='1' then
					s <= 8;
				else
					s <= 7;
				end if;
			elsif s=8 then		 						-- put second nibble out
					s <= 9;
			elsif s=9 then								-- set second nibble timer
					s <= 10;
			elsif s=10 then							-- enable second nibble write
				if tim_done='1' then
					s <= 11;
				else
					s <= 10;
				end if;
			elsif s=11 then							-- wait 20ns
					s <= 12;								-- byte done
			elsif s=12 then
					s <=0;
			else 
					s <=0;
			end if;
--****************************************************************
--timer_20bit
			if set_timer_t1 = '1' then
				t <= timer_value_t1;
			elsif set_timer_t2 = '1' then
				t <= timer_value_t2;
			elsif set_timer_t3 = '1' then
				t <= timer_value_t3;
			elsif t /= "00000000000000000000" then
				t <= t-1;
			else
				t <= t;
			end if;
--****************************************************************
		end if;
	end process;
--****************************************************************
--internal signals	
	tim_done <= '1'when t = "00000000000000000001" else '0';
	with q select
		timer_value_t1 <= "00000000000000000000" when "000000",
								T_15ms when "000001",
								T_100us when "000010",
								T_1_64ms when "001000",
								T_40us when others;
	with q select
		timer_value_t3 <= "00000000000000000000" when "000000",
								T_4_1ms when "000001",
								T_40us when others;
		timer_value_t2 <= T_1us;

	with q select
		byte <=  "00000000" when "000000",
					"00110011" when "000001",
					"00110010" when "000010",
					"00101000" when "000011",
					"00000110" when "000100",
					"00001100" when "000101",
					"00000000" when "000110",
					"00000001" when "000111",
					"10101000" when "011000",
					ascii when others;	

	enable_byte_write <= '0' when ((q="000000")or(q="000110"))else'1';

	set_timer_t1 <='1' when (s=1) else '0';
	set_timer_t2 <='1' when (s=4)or(s=9) else '0';
	set_timer_t3 <='1' when (s=6) else '0';
	byte_done <= '1' when s=11 else '0';
--**************************************************************************************
--outputs
	next_char <= '1' when (((q(5)='1')or(q(4)='1')or(q(3)='1'))and(q/="011000"))and(byte_done='1')else'0';	
	lcd_rs <= 	'0' when (q="000000")or(q="000001")or(q="000010")or(q="000011")or(q="000100")or
									(q="000101")or(q="000110")or(q="000111")or(q="011000")else'1';	
	lcd_rw <= '0'; 
	lcd_e <= '1' when (s=5)or(s=10) else '0';
	nibble_to_lcd(3) <=  '1' when ((byte(7)='1')and((s=3)or(s=4)or(s=5)or(s=6)))or 
						  				  ((byte(3)='1')and((s=8)or(s=9)or(s=10)or(s=11))) else '0';
	nibble_to_lcd(2) <=  '1' when ((byte(6)='1')and((s=3)or(s=4)or(s=5)or(s=6)))or 
						  				  ((byte(2)='1')and((s=8)or(s=9)or(s=10)or(s=11))) else '0';
	nibble_to_lcd(1) <=  '1' when ((byte(5)='1')and((s=3)or(s=4)or(s=5)or(s=6)))or 
						  				  ((byte(1)='1')and((s=8)or(s=9)or(s=10)or(s=11))) else '0';
	nibble_to_lcd(0) <=  '1' when ((byte(4)='1')and((s=3)or(s=4)or(s=5)or(s=6)))or 
						  				  ((byte(0)='1')and((s=8)or(s=9)or(s=10)or(s=11))) else '0';										  
--**************************************************************************************
end Behavioral;

