-- 
-- Create Date:    20:32:45 10/04/2010 
-- Module Name:    tx_rx_module - Behavioral 
-- Description:    Data hub for tx og rx, send ut 3 tx data byte p serie-form, vent p rx data for  lage parallell-form rx data byte
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity tx_rx_module is
    Port ( spd, dir, msc: in  STD_LOGIC_VECTOR (7 downto 0);
           rx, timer_expired, error, update, clk_4b : in  STD_LOGIC;
           rx_byte : out  STD_LOGIC_VECTOR (7 downto 0);
           tx, linestatus, start_timer, stop_timer : out  STD_LOGIC);
end tx_rx_module;

architecture Behavioral of tx_rx_module is
		signal a : INTEGER RANGE 0 TO 5000:=1; -- 123 sendemodus, 39 mottaker, pluss rx mellom 0 og 25 ms
		signal rx_ref_pulse : INTEGER RANGE 0 TO 5000; -- referanse puls for rx del, pluss rx mellom 0 og 25 ms
begin

process (clk_4b, a, update, rx, error, timer_expired)
   begin
		if (clk_4b'EVENT and clk_4b = '1') then

			if (a = 0) then
					stop_timer <= '0';
					a <= a+1; -- tell opp global pulseteller til 1

			elsif (a = 1 and update = '1') then
					linestatus <= '1';
					a <= a+1; -- tell opp global pulseteller til 2

			elsif (a >= 2 and a <= 121) then

				-- retning
				if (a >= 2 and a <= 5) then
					tx <= '0'; -- startbit
				elsif (a >= 6 and a <= 37) then
					-- loop gjennom retning byte
					for i in 0 to 7 loop
						if (a >= 2+(4*(i+1)) and a <= 2+(4*(i+1))+3) then
							tx <= dir(i);
						end if;
					end loop;
				elsif a >= 38 and a <= 41 then
					tx <= '1'; -- stoppbit
				end if;

				-- hastighet
				if (a >= 42 and a <= 45) then
					tx <= '0'; -- startbit
				elsif (a >= 46 and a <= 77) then
					-- loop gjennom hastighet byte
					for i in 0 to 7 loop
						if (a >= 42+(4*(i+1)) and a <= 42+(4*(i+1))+3) then
							tx <= spd(i);
						end if;
					end loop;
				elsif a >= 78 and a <= 81 then
					tx <= '1'; -- stoppbit
				end if;

				-- diverse
				if (a >= 82 and a <= 85) then
					tx <= '0'; -- startbit
		
				elsif (a >= 86 and a <= 117) then
					-- loop gjennom misc byte
					for i in 0 to 7 loop
						if (a >= 82+(4*(i+1)) and a <= 82+(4*(i+1))+3) then
							tx <= msc(i);
						end if;
					end loop;

				elsif a >= 118 and a <= 121 then
					tx <= '1'; -- stoppbit
				end if;
				
				a <= a+1; -- tell opp global pulseteller


			elsif (a = 122) then

				-- klargjrelse for mottakelse
				linestatus <= '0';
				start_timer <= '1';
				a <= a+1; -- tell opp global pulseteller

			elsif (a >= 123) then

				start_timer <= '0'; -- rx_timer allerede startet
			
				-- vemter p response fra styrenhet, opp til 25 ms
				if (rx = '0' and rx_ref_pulse = 0) then			
					-- mottat frste startbit fra bil
					-- fant frste rx 0 startbit, sett opp en start_puls_vektor, f.eks puls 132 brukes som referanse for  lese ut resten av bitene
					rx_ref_pulse <= a;
					stop_timer <= '1';

				elsif (rx_ref_pulse >= 1) then
					stop_timer <= '0';
				
					if (a >= rx_ref_pulse and a <= (rx_ref_pulse+3)) then
						-- startbit, resten
					elsif (a >= (rx_ref_pulse+4) and a <= (rx_ref_pulse+35)) then

						-- loop gjennom return byte
						for i in 0 to 7 loop
							-- les ut midterste klokkebit
							if (a >= (rx_ref_pulse+(4*(i+1))+2) and a <= (rx_ref_pulse+(4*(i+1))+3)) then
									rx_byte(i) <= rx;
							end if;
						end loop;
				
					elsif (a >= (rx_ref_pulse+36) and a <= (rx_ref_pulse+39)) then
						-- stoppbit, rx settes hy resten av tiden
					end if;
				end if;

				-- timer_expired eller kommunikasjonsfeil
				if (error = '1' or timer_expired = '1' or (rx_ref_pulse >= 1 and a >= (rx_ref_pulse+40))) then
						stop_timer <= '1';
						rx_ref_pulse <= 0;
						linestatus <= '1'; -- tx modus
						a <= 0;
				else 
						a <= a+1; -- tell opp global pulseteller
				end if;
				
			end if; -- end rx

		end if;
	end process;

end Behavioral;

